"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Api = exports.ApiError = void 0;
const undici_1 = require("undici");
const debug = require("debug");
const fs_1 = require("fs");
class ApiError extends Error {
}
exports.ApiError = ApiError;
class Api {
    constructor(token, client) {
        this.token = token;
        this.log = debug("tg-io:api");
        this.baseUrl = `https://api.telegram.org`;
        this.clientOptions = { keepAliveTimeout: 6e3 };
        this.client = new undici_1.Client(this.baseUrl);
        this.client = client !== null && client !== void 0 ? client : this.client;
        this.client.on("connect", () => this.log("keep-alive connection created"));
        this.client.on("disconnect", () => this.log("keep-alive connection closed"));
    }
    /**
     * Call method with params as a formdata
     */
    async upload(method, params) {
        var _a, _b;
        if (!(params instanceof undici_1.FormData)) {
            let formdata = new undici_1.FormData();
            for (let [key, value] of Object.entries(params)) {
                let filename = null;
                if (Buffer.isBuffer(value)) {
                    filename = "file.data";
                }
                else if (value instanceof fs_1.ReadStream &&
                    fs_1.ReadStream.isReadable(value)) {
                    filename = (_b = (_a = value.path) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : "file.data";
                }
                formdata.append(key, value, filename);
            }
            params = formdata;
        }
        return this.callMethod(method, params);
    }
    async callMethod(method, params) {
        let response;
        try {
            response = await this.client.request(Object.assign(Object.assign({ method: "POST", path: `/bot${this.token}/${method}` }, this.parseParams(params)), { throwOnError: true }));
        }
        catch (err) {
            this.handleTelegramApiError(err, method, JSON.stringify(params));
        }
        finally {
            this.log(`${method} ${response ? "OK" : "ERROR"}`);
        }
        return (await response.body.json());
    }
    parseParams(params) {
        if (params instanceof undici_1.FormData) {
            return {
                body: params,
            };
        }
        let body = JSON.stringify(params);
        return {
            body,
            headers: {
                "Content-Type": "application/json",
                "Content-Length": body.length.toString(),
            },
        };
    }
    handleTelegramApiError(err, method, params) {
        if (!(err instanceof undici_1.errors.ResponseStatusCodeError)) {
            throw err;
        }
        let body = undefined;
        if (typeof err.body === "string") {
            try {
                body = JSON.parse(err.body);
            }
            catch (_) {
                body = undefined;
            }
        }
        else {
            body = err.body;
        }
        if (!body) {
            throw err;
        }
        let errorData = body;
        let error = new ApiError(errorData.description);
        error.code = errorData.error_code;
        error.method = method;
        error.params = params;
        throw error;
    }
}
exports.Api = Api;
